# coding: utf-8
require 'sinatra'
require 'base64'
require 'open-uri'
require 'cgi'
require 'openssl'
require 'json'

# 请填写您的AccessKeyId。
$access_key_id = '<yourAccessKeyId>'
# 请填写您的AccessKeySecret。
$access_key_secret = '<yourAccessKeySecret>'
# $host的格式为 bucketname.endpoint ，请替换为您的真实信息。
$host = 'http://bucket-name.oss-cn-hangzhou.aliyuncs.com';
# $callbackUrl为 上传回调服务器的URL，请将下面的IP和Port配置为您自己的真实信息。
$callback_url = "http://88.88.88.88:8888";
# 用户上传文件时指定的前缀。
$upload_dir = 'user-dir-prefix/'
$expire_time = 30

$server_ip = "0.0.0.0"
$server_port = 8080
if $*.length == 1 
  if not $*[0].nil? then $server_port=$*[0] end
elsif $*.length == 2
  if not $*[0].nil? then $server_ip=$*[0] end
  if not $*[1].nil? then $server_port=$*[1] end
end
puts "App server is running on : http://#{$server_ip}:#{$server_port} "

# 接受OSS上传回调的server示例，利用RSA公钥验证请求来自OSS，而非其
# 他恶意请求。具体签名/验证过程请参考：
# https://help.aliyun.com/document_detail/oss/api-reference/object/Callback.html

def get_header(name)
  key = "http_#{name.gsub('-', '_')}".upcase
  request.env[key]
end

PUB_KEY_URL_PREFIX = 'http://gosspublic.alicdn.com/'
PUB_KEY_URL_PREFIX_S = 'https://gosspublic.alicdn.com/'

# 1. Public key is cached so that we don't need fetching it for every request
# 2. Ensure pub_key_url is an authentic URL by asserting it starts
#    with the offical domain
def get_public_key(pub_key_url, reload = false)
  unless pub_key_url.start_with?(PUB_KEY_URL_PREFIX) ||
         pub_key_url.start_with?(PUB_KEY_URL_PREFIX_S)
    fail "Invalid public key URL: #{pub_key_url}"
  end

  if reload || @pub_key.nil?
    @pub_key = open(pub_key_url) { |f| f.read }
  end

  @pub_key
end

def hash_to_jason(source_hash)
    jason_string = source_hash.to_json;	
	
    jason_string.gsub!("\":[", "\": [")
    jason_string.gsub!("\",\"", "\", \"")
    jason_string.gsub!("],\"", "], \"")
    jason_string.gsub!("\":\"", "\": \"")
	
    jason_string
end

def get_token()
    expire_syncpoint = Time.now.to_i + $expire_time
	
    expire = Time.at(expire_syncpoint).utc.iso8601()
    response.headers['expire'] = expire
	
    policy_dict = {}
    condition_arrary = Array.new
    array_item = Array.new
    array_item.push('starts-with')
    array_item.push('$key')
    array_item.push($upload_dir)
    condition_arrary.push(array_item)
    policy_dict["conditions"] = condition_arrary
    policy_dict["expiration"] = expire
    policy = hash_to_jason(policy_dict)
    policy_encode = Base64.strict_encode64(policy).chomp;
    h = OpenSSL::HMAC.digest('sha1', $access_key_secret, policy_encode)
    hs = Digest::MD5.hexdigest(h)
    sign_result = Base64.strict_encode64(h).strip()
	
    callback_dict = {}
    callback_dict['callbackBodyType'] = 'application/x-www-form-urlencoded';
    callback_dict['callbackBody'] = 'filename=${object}&size=${size}&mimeType=${mimeType}&height=${imageInfo.height}&width=${imageInfo.width}';
    callback_dict['callbackUrl'] = $callback_url;
    callback_param = hash_to_jason(callback_dict)
    base64_callback_body = Base64.strict_encode64(callback_param);
	
    token_dict = {}
    token_dict['accessid'] = $access_key_id
    token_dict['host'] = $host
    token_dict['policy'] = policy_encode
    token_dict['signature'] = sign_result 
    token_dict['expire'] = expire_syncpoint
    token_dict['dir'] = $upload_dir
    token_dict['callback'] = base64_callback_body
    response.headers["Access-Control-Allow-Methods"] = "POST"
    response.headers["Access-Control-Allow-Origin"] = "*"
    result = hash_to_jason(token_dict)
    
    result
end

set :bind, $server_ip
set :port, $server_port

get '/*' do
  puts "********************* GET "
  get_token()
end

post '/*' do
  puts "********************* POST"
  pub_key_url = Base64.decode64(get_header('x-oss-pub-key-url'))
  pub_key = get_public_key(pub_key_url)
  rsa = OpenSSL::PKey::RSA.new(pub_key)

  authorization = Base64.decode64(get_header('authorization'))
  req_body = request.body.read
  if request.query_string.empty? then
    auth_str = CGI.unescape(request.path) + "\n" + req_body
  else
    auth_str = CGI.unescape(request.path) + '?' + request.query_string + "\n" + req_body
  end

  valid = rsa.public_key.verify(
    OpenSSL::Digest::MD5.new, authorization, auth_str)

  if valid
    #body({'Status' => 'OK'}.to_json)
    body(hash_to_jason({'Status' => 'OK'}))
  else
    halt 400, "Authorization failed!"
  end
end
